require "bunny"
require "blamescope/version"
require "blamescope/middleware"
require "blamescope/trackable"
require "blamescope/server"

# 
# Blamescope module
# 
# @author Volodymyr Ladnik
# 
module Blamescope
  class << self
    attr_accessor :exchange

    def included(base)
      base.extend(Trackable)
      base.class.extend(Trackable)
    end

    # 
    # Configure Blamescope
    # @param [Hash] config
    # @option config [String] :rabbit Rabbit connection in format amqp://user:pass@host:port/vhost
    # 
    def configure(config)
      self.exchange = rabbit_connection(config[:rabbit])[:exchange]
    end

    # 
    # Get current user
    # 
    # @return [String] user email
    def user
      env = Thread.current[:env]
      if env && env['warden']
        env['warden'].user.email
      else
         "#{ENV['USER']}@#{Socket.gethostname}"
      end
    end

    # 
    # Send event
    # @param [type] attrs  event attributes
    # @option attrs [String] :model current class name
    # @option attrs [String] :action current method
    # @option attrs [String] :attrs object attributes
    # 
    def emit(attrs)
      attrs[:email] ||= user
      exchange.publish(attrs.to_json)
    rescue Bunny::ConnectionClosedError
      puts "Bunny is down: couldn't send #{attrs[:action]} action for #{attrs[:model]} model"
    end

    # 
    # Create RabbitMQ connection
    # @param config [String] Rabbit connection in format amqp://user:pass@host:port/vhost
    # 
    # @return [Hash] rabbit connection, channel and exchange
    def rabbit_connection(config)
      conn = Bunny.new(config)
      conn.start
      channel = conn.create_channel
      exchange = channel.fanout("blamescope")
      {conn: conn, channel: channel, exchange: exchange}
    end
  end
end
