# 
# Trackable module
# 
# @author Volodymyr Ladnik
# 
module Blamescope
  module Trackable

    # 
    # Send event for model
    # @param model [Object] current object
    # @param method_name [String|Symbol] method invoked
    # 
    # @return [type] [description]
    def self.send_event(model, method_name)
      class_name = model.class == Class ? model.name : model.class.name
        attrs = {}
        attrs[:id] = model.id if model.respond_to? :id
        if model.respond_to? :name
          attrs[:name] = model.name
        elsif model.respond_to? :title
          attrs[:title] = model.title
        end
        Blamescope.emit(model: class_name, action: method_name, attrs: attrs)
    end

    # 
    # Track method
    # @param name [String|Symbol] method name
    # 
    def blame_method method_name
      alias_method "original_#{method_name}", method_name
      define_method method_name do |*args, &block|
        Blamescope::Trackable.send_event(self, method_name)
        send("original_#{method_name}", *args, &block)
      end
    end

    # 
    # Track multiple methods
    # @param *names [String|Symbol] method name
    # 
    def blame_methods *names
      names.each do |name|
        blame_method(name)
      end
    end

    # 
    # Track ActiveRecord model
    # 
    def blame_activerecord
      after_create do
        Blamescope::Trackable.send_event(self, :create)
      end
      after_update do
        Blamescope::Trackable.send_event(self, :update)
      end
      after_destroy do
        Blamescope::Trackable.send_event(self, :delete)
      end
    end
  end
end
